@@ -10,3 +10,8 @@ def updateData(data,**kwargs):
   return data
 def getCallArgs(endpoint):
   return {'getMetaData': ['signature'], 'getPoolData': ['signature'], 'getTransactionData': ['signature'], 'getPoolInfo': ['signature'], 'getMarketInfo': ['signature'], 'getKeyInfo': ['signature'], 'getLpKeys': ['signature'], 'process': ['signature']}.get(get_endpoint(endpoint))
+def ifListGetSection(listObj,section=0):
+    if isinstance(listObj,list):
+        if len(listObj)>section:
+            return listObj[section]
+    return listObj
